<?php
namespace OCA\IfcViewer\Controller;

use OCP\AppFramework\Http\DataResponse;
use OCP\AppFramework\Http\RedirectResponse;
use OCP\AppFramework\Http\DownloadResponse;
use OCP\AppFramework\Http\ContentSecurityPolicy;
use OCP\AppFramework\Http\TemplateResponse;
use OCP\AppFramework\Http\Template\PublicTemplateResponse;
use OCP\AppFramework\Http\DataDownloadResponse;
use OCP\AppFramework\Http\JSONResponse;
use OCP\AppFramework\Controller;
use OCP\AutoloadNotAllowedException;
use OCP\Constants;
use OCP\Files\FileInfo;
use OCP\Files\IRootFolder;
use OCP\ILogger;
use OCP\IRequest;
use OCP\ISession;
use OCP\IURLGenerator;
use OCP\IUserSession;
use OCP\Share\Exceptions\ShareNotFound;
use OCP\Share\IManager;
use OC\Files\Filesystem;
use OC\Files\View;
use OC\User\NoUserException;
use OCA\Files\Helper;
use OCA\Files_Versions\Storage;
use OCA\Viewer\Event\LoadViewer;
use OCA\IfcViewer\AppConfig;

class IfcviewerController extends Controller
{

    private $userSession;

    private $root;

    private $urlGenerator;

    private $logger;

    private $config;

    /**
     * Session
     *
     * @var ISession
     */
    private $session;

    /**
     * Share manager
     *
     * @var IManager
     */
    private $shareManager;

    public function __construct($AppName, IRequest $request, IRootFolder $root, IUserSession $userSession, IURLGenerator $urlGenerator, ILogger $logger, IManager $shareManager, ISession $session)
    {
        parent::__construct($AppName, $request);

        $this->userSession = $userSession;
        $this->root = $root;
        $this->urlGenerator = $urlGenerator;
        $this->logger = $logger;
        $this->config = new AppConfig($AppName);
        $this->shareManager = $shareManager;
        $this->session = $session;
    }

    /**
     *
     * @NoAdminRequired
     * @NoCSRFRequired
     * @PublicPage
     */
    public function public($fileId, $shareToken)
    {
        return $this->index($fileId, $shareToken, null, true);
    }

    /**
     *
     * @NoAdminRequired
     * @NoCSRFRequired
     * @PublicPage
     */
    public function publicindex($fileId, $shareToken)
    {
        return $this->index($fileId, $shareToken, null, true);
    }

    /**
     *
     * @NoAdminRequired
     * @NoCSRFRequired
     * @PublicPage
     */
    public function bimit($projectUuid, $hash)
    {
        $frameUrl = $this->config->GetFrameUrl();
        $redirectUrl = frameUrl + "/project/cloud/" . $projectUuid . "/" . $hash;
        return new RedirectResponse($redirectUrl);
    }

    /**
     *
     * @NoAdminRequired
     * @NoCSRFRequired
     */
    public function index($fileId, $shareToken = NULL, $filePath = NULL, $publicView = false)
    {
        if (empty($shareToken) && ! $this->userSession->isLoggedIn()) {
            $redirectUrl = $this->urlGenerator->linkToRoute("core.login.showLoginForm", [
                "redirect_url" => $this->request->getRequestUri()
            ]);
            return new RedirectResponse($redirectUrl);
        }
        
        $bimitUrl = $this->config->GetBimitUrl();
        $bimitPath = $this->config->GetBimitPath();
        $frameUrl = $this->config->GetFrameUrl();

        $nameOfFile = null;
        $relativePath = null;
        $filesystemPath = null;
        $response = array();
        if (empty($shareToken)) {
            list ($file, $error) = $this->getFile($fileId);
            if (isset($error)) {
                $this->logger->error("Load with id: " . $fileId . " " . $error, array(
                    "app" => $this->appName
                ));
                $response = array_merge($response, array(
                    "httpCode" => 0,
                    "error" => "get_file_by_id",
                    "error_description" => $error
                ));
                return $this->renderError($response);
            }
            $nameOfFile = $file->getName();

            // $uid = $this->userSession->getUser()->getUID();
            // $baseFolder = $this->root->getUserFolder($uid);
            $uid = $file->getOwner()->getUID();
            $baseFolder = $this->root->getUserFolder($uid);
            $relativePath = $baseFolder->getRelativePath($file->getPath());
            $filesystemPath = $this->getOnlyFile($baseFolder->getRelativePath($file->getPath()), $uid);
        } else {
            list ($file, $error, $share) = $this->getFileByToken($fileId, $shareToken);
            if (isset($error)) {
                $this->logger->error("Load with token: " . $shareToken . " " . $error, array(
                    "app" => $this->appName
                ));
                $response = array_merge($response, array(
                    "httpCode" => 0,
                    "error" => "get_file_by_token",
                    "error_description" => $error
                ));
                return $this->renderError($response);
            }

            $nameOfFile = $file->getName();
            
            $uid = $file->getOwner()->getUID();
            $baseFolder = $this->root->getUserFolder($uid);
            $relativePath = $baseFolder->getRelativePath($file->getPath());
            $filesystemPath = $this->getOnlyFile($baseFolder->getRelativePath($file->getPath()), $uid);
        }
        if (file_exists($filesystemPath)) {
            $hash = md5_file($filesystemPath);
            $hash = hash("md5", $filesystemPath . $hash);

            $data = [
                "ifcFilePath" => rawurlencode($relativePath),
                "title" => $nameOfFile,
                "annotation" => "",
                "hash" => $hash,
                "fileSize" => filesize($filesystemPath),
                "loadingUrl" => $bimitUrl . $bimitPath . "project",

                "frameUrl" => $frameUrl,
                "bimitUrl" => $bimitUrl,
                "fileId" => $fileId,
                "filePath" => rawurlencode($relativePath),
                "shareToken" => $shareToken
            ];

            if ($publicView) {
                $response = new PublicTemplateResponse($this->appName, "openifc", $data);
                $response->setHeaderTitle($nameOfFile);
            } else {
                //$response = new TemplateResponse($this->appName, "openifc", $data, 'blank');
                $response = new TemplateResponse($this->appName, "openifc", $data);
                //$response = new PublicTemplateResponse($this->appName, "openifc", $data);
            }

            $csp = new ContentSecurityPolicy();
            $csp->allowInlineScript(true);
            $csp->allowEvalScript(true);

            if (isset($bimitUrl) && ! empty($bimitUrl)) {
                $csp->addAllowedScriptDomain("*");
                // $csp->addAllowedStyleDomain("*");
                $csp->addAllowedFormActionDomain("*");
                $csp->addAllowedConnectDomain("*");
                $csp->addAllowedMediaDomain("*");
                $csp->addAllowedObjectDomain("*");
                $csp->addAllowedFrameDomain("*");
                $csp->addAllowedFrameDomain("blob:");
                $csp->addAllowedChildSrcDomain("*");
                $csp->addAllowedChildSrcDomain("blob:");
            }
            $response->setContentSecurityPolicy($csp);

            return $response;
        } else {
            $this->logger->error("Can't find file at " . $relativePath, array(
                "app" => $this->appName
            ));

            $response = array_merge($response, array(
                "httpCode" => 0,
                "error" => "file_not_found",
                "error_description" => "Can't find file " . ($fileId ? "by id " . $fileId : "by token " . $shareToken)
            ));
            return $this->renderError($response);
        }
    }

    /*
     * public function getFile($directory, $fileName)
     * {
     * \OC_Util::tearDownFS();
     * \OC_Util::setupFS($this->userId);
     * return Filesystem::getLocalFile($directory . '/' . $fileName);
     * }
     */
    public function getOnlyFile($fileName, $ownerFile)
    {
        \OC_Util::tearDownFS();
        \OC_Util::setupFS($ownerFile);
        return Filesystem::getLocalFile($fileName);
    }

    /**
     *
     * @NoAdminRequired
     * @NoCSRFRequired
     * @PublicPage
     */
    public function sendIfc()
    {
        $bimitUrl = $this->config->GetBimitUrl();
        $bimitPath = $this->config->GetBimitPath();

        $filesystemPath = null;
        $fileId = trim($_POST['fileId']);
        $shareToken = trim($_POST['shareToken']);
        // $relativePath = trim($_POST['relativePath']);
        $modelUuid = trim($_POST['modelUuid']);

        $response = array();
        if (empty($shareToken)) {
            list ($file, $error) = $this->getFile($fileId);
            if (isset($error)) {
                $this->logger->error("Load with id: " . $fileId . " " . $error, array(
                    "app" => $this->appName
                ));
                $response = array_merge($response, array(
                    "httpCode" => 0,
                    "error" => "get_file_by_id",
                    "error_description" => $error
                ));
                return $this->renderError($response);
            }
            $nameOfFile = $file->getName();
            $uid = $file->getOwner()->getUID();
            $baseFolder = $this->root->getUserFolder($uid);
            $relativePath = $baseFolder->getRelativePath($file->getPath());
            $filesystemPath = $this->getOnlyFile($baseFolder->getRelativePath($file->getPath()), $uid);
        } else {
            list ($file, $error, $share) = $this->getFileByToken($fileId, $shareToken);
            if (isset($error)) {
                $this->logger->error("Load with token: " . $shareToken . " " . $error, array(
                    "app" => $this->appName
                ));
                $response = array_merge($response, array(
                    "httpCode" => 0,
                    "error" => "get_file_by_token",
                    "error_description" => $error
                ));
                return $this->renderError($response);
            }

            $nameOfFile = $file->getName();

            $uid = $file->getOwner()->getUID();
            $baseFolder = $this->root->getUserFolder($uid);
            $relativePath = $baseFolder->getRelativePath($file->getPath());
            $filesystemPath = $this->getOnlyFile($baseFolder->getRelativePath($file->getPath()), $uid);
        }
        if (file_exists($filesystemPath)) {
            $this->logger->info("SEND FILE " . $nameOfFile . " TO THE " . $bimitUrl . $bimitPath . "revision");

            $cfile = new \CURLFile($filesystemPath, 'application/x-step', $nameOfFile);

            $postData = array(
                'modelUuid' => $modelUuid,
                'file' => $cfile
            );

            $data = $this->CurlToHost($bimitUrl . $bimitPath . "revision", $postData);

            $this->logger->info($data);

            $dataObj = json_decode($data);
            $httpCode = $dataObj->httpCode;
            if ($httpCode >= 200 && $httpCode < 300) {
                $response = array_merge($response, array(
                    "httpCode" => 1
                ));
                return json_encode($response);
            } else
                return $data;
        } else {
            $this->logger->error("Can't find file at " . $relativePath, array(
                "app" => $this->appName
            ));

            $response = array_merge($response, array(
                "httpCode" => 0,
                "error" => "file_not_found",
                "error_description" => "Can't find file " . $relativePath
            ));
            return json_encode($response);
        }
    }

    /**
     *
     * @NoAdminRequired
     */
    private function getFile($fileId)
    {
        if (empty($fileId)) {
            return [
                null,
                "FileId is empty"
            ];
        }

       
        $files = $this->root->getById($fileId);
        if (empty($files)) {
            return [
                null,
                "File not found by id"
            ];
        }
        $file = $files[0];

        if (! $file->isReadable()) {
            return [
                null,
                "You do not have enough permissions to view the file"
            ];
        }
        
        $uid = $file->getOwner()->getUID();
        $baseFolder = $this->root->getUserFolder($uid);
        $files = $baseFolder->getById($fileId);
        
        if (empty($files)) {
            return [
                null,
                "File not found by id"
            ];
        }
        $file = $files[0];
        
        if (! $file->isReadable()) {
            return [
                null,
                "You do not have enough permissions to view the file"
            ];
        }
        
        return [
            $file,
            null
        ];
    }

    /**
     * Getting file by token
     *
     * @param integer $fileId
     *            - file identifier
     * @param string $shareToken
     *            - access token
     *            
     * @return array
     */
    private function getFileByToken($fileId, $shareToken)
    {
        list ($node, $error, $share) = $this->getNodeByToken($shareToken);

        if (isset($error)) {
            return [
                NULL,
                $error,
                NULL
            ];
        }

        //$this->logger->info("" . $node);
        if ($node instanceof \OC\Files\Node\Folder) {
            try {
                $files = $node->getById($fileId);
            } catch (\Exception $e) {
                $this->logger->error("getFileByToken: $fileId " . $e->getMessage(), array(
                    "app" => $this->appName
                ));
                return [
                    NULL,
                    "Invalid request",
                    NULL
                ];
            }

            if (empty($files)) {
                $this->logger->info("Files not found: $fileId", array(
                    "app" => $this->appName
                ));
                return [
                    NULL,
                    "File not found by token",
                    NULL
                ];
            }
            $file = $files[0];
        } else {
            $file = $node;
        }

        return [
            $file,
            NULL,
            $share
        ];
    }

    /**
     * Getting file by token
     *
     * @param string $shareToken
     *            - access token
     *            
     * @return array
     */
    private function getNodeByToken($shareToken)
    {
        list ($share, $error) = $this->getShare($shareToken);

        if (isset($error)) {
            return [
                NULL,
                $error,
                NULL
            ];
        }

        if (($share->getPermissions() & Constants::PERMISSION_READ) === 0) {
            return [
                NULL,
                "You do not have enough permissions to view the file",
                NULL
            ];
        }

        try {
            $node = $share->getNode();
        } catch (NotFoundException $e) {
            $this->logger->error("getFileByToken error: " . $e->getMessage(), array(
                "app" => $this->appName
            ));
            return [
                NULL,
                "File not found by node token",
                NULL
            ];
        }

        return [
            $node,
            NULL,
            $share
        ];
    }

    /**
     * Getting share by token
     *
     * @param string $shareToken
     *            - access token
     *            
     * @return array
     */
    private function getShare($shareToken)
    {
        if (empty($shareToken)) {
            return [
                NULL,
                "FileId is empty"
            ];
        }

        $share;
        try {
            $share = $this->shareManager->getShareByToken($shareToken);
        } catch (ShareNotFound $e) {
            $this->logger->error("getShare error: " . $e->getMessage(), array(
                "app" => $this->appName
            ));
            $share = NULL;
        }

        if ($share === NULL || $share === false) {
            return [
                NULL,
                "You do not have enough permissions to view the file"
            ];
        }

        if ($share->getPassword() && (! $this->session->exists("public_link_authenticated") || $this->session->get("public_link_authenticated") !== (string) $share->getId())) {
            return [
                NULL,
                "You do not have enough permissions to view the file"
            ];
        }

        return [
            $share,
            NULL
        ];
    }

    private function CurlToHost($path, $postdata)
    {
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $path);
        curl_setopt($ch, CURLOPT_HEADER, false);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_POST, true);
        // curl_setopt($ch, CURLOPT_POSTFIELDS, $file);
        // $post = array('val1' => 'value','val2' => 'value','file_contents'=> $curlFile );
        curl_setopt($ch, CURLOPT_POSTFIELDS, $postdata);

        $data = curl_exec($ch);

        $this->logger->info($data);

        $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);

        curl_close($ch);

        $data = json_decode($data, true);

        if ($data == null) {

            $data = array();

            $data = array_merge($data, array(
                "httpCode" => $httpCode,
                "error" => "unckhown_error",
                "error_description" => "Ошибка обработки данных"
            ));
            return json_encode($data);
        }

        $data = array_merge($data, array(
            "httpCode" => $httpCode
        ));

        return json_encode($data);
    }

    private function renderError($error)
    {
        return new TemplateResponse("", "error", [
            "errors" => [
                [
                    "error" => '', // $error['error'],
                    "hint" => $error['error_description']
                ]
            ]
        ], "error");
    }
}
